/* Language Includes */
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>

/* Standard System Include */
#include <unistd.h>
#include <time.h>

#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>

#include "game.h"
#include "initialisation.h"
#include "deroulement.h"
#include "manches.h"

#define SCREENWIDTH  640
#define SCREENHEIGH  480
#define BPP  32

#define PLAYERW  160
#define PLAYERH  120

#define  BUTTONWIDTH 81
#define  BUTTONHEIGH 22

#define SPIRITWIDTH  48
#define SPIRITHEIGH  48

SDL_Surface *gpScreen = NULL;//显示表面
SDL_Surface *gpPLAYER = NULL;//图片表面
SDL_Surface *gpBackGround = NULL;//背景图片
SDL_Surface *gpButton = NULL;
SDL_Surface *gpSpirit;
SDL_Event myEvent;

SDL_Surface *loadImage(char *aFilename);
void cleanUp();//程序退出时清理内存
int isOnButton(int aX,int aY,int aIndex);
void changeButton(BUTTONPOS aPos,int aNo,SDL_Rect *aDst);
char *localeToUTF8(char *dice);

typedef enum {LEFT,RIGHT}BUTTONPOS;//枚举按钮在精灵图中的编号
typedef enum {1,2,OTHER}BUTTONFLAG;//枚举按钮


int main(int argc,char *argv[])
{
    srand(time(NULL));
    char a[] = "Bienvenu! Cher joueur!";
    char b[] = "How many players";
    wchar_t * p ;

    int quit = 0;
    int PLAYERX = 240;//初始坐标
    int PLAYERY = 320;

    int x,y;
    SDL_Rect src,dst;
    //SDL_Rect dice[2][3];
    //Uint32 colorkey;
    BUTTONFLAG selected = OTHER;//按钮选中标志
    int current=-1;

    int i,j;
    int spiritX = 250;//dice的初始坐标
    int spiritY = 330;

    if((SDL_Init(SDL_INIT_VIDEO)==-1)) //初始化视频子系统
    {
        printf("Unable to init SDL: %s\n", SDL_GetError());
        exit(-1);
    }
    atexit(SDL_Quit);// 注册SDL_Quit，当退出时调用，使得退出时程序自动清理

    if(!init("WORD","icon.bmp"))
    {
       exit(0);
    }

    //创建32位640*480窗口
    gpScreen = SDL_SetVideoMode(SCREENWIDTH,SCREENHEIGH, BPP, SDL_HWSURFACE | SDL_HWPALETTE | SDL_DOUBLEBUF );
    if(!gpScreen)
    {
        exit(1);
    }

    //打开simfang.ttf 字库，设字体为20号
     font  = TTF_OpenFont("simhei.ttf",20);
     if(font == NULL)
     {
        fprintf(stderr,"font open failure %s\n",SDL_GetError());
        exit(-1);
     }

    gpPLAYER = loadImage("PLAYER.bmp");
    gpBackGround = loadImage("background.jpg");
    gpBackground = loadImage("ressources\\test.bmp");
    gpButton = loadImage("ressources\\1.bmp");
    //gpSpirit = loadImage("ressources\\dice.bmp");//载入精灵大图

    applySurface(0,0,gpBackground,gpScreen);
    gpMessage[0] = TTF_RenderText_Solid(font,a,RGB_Black);
    gpMessage[1] = TTF_RenderText_Solid(font,a,RGB_Black);
    applySurface(80,120,gpMessage[0],gpScreen);
    applySurface(80,150,gpMessage[1],gpScreen);

    //SDL_BlitSurface(gpBackGround,NULL,gpScreen,NULL);
    SDL_BlitSurface(gpPLAYER,NULL,gpScreen,NULL);
    SDL_Flip(gpScreen);
    SDL_EnableKeyRepeat(500,30);

    TTF_CloseFont(font);

    //colorkey= SDL_MapRGB(gpSpirit->format, 255, 238, 187);
    //SDL_SetColorKey(gpSpirit, SDL_SRCCOLORKEY , colorkey );


    if (!gpBackground||!gpButton)
    {
        exit(0);
    }

    for (i = 0;i<2;i++) //设定每一个精灵在精灵大图上的位置
    {
        for (j = 0;j<3;j++)
        {
            src[i][j].x = j*SPIRITWIDTH;
            src[i][j].y = i*SPIRITHEIGH;
            src[i][j].w = SPIRITWIDTH;
            src[i][j].h = SPIRITHEIGH;
        }
    }
    /*
    dst.x=dst.y = 0;
    SDL_BlitSurface(gpSpirit,&src[0][0],gpScreen,&dst);//在屏幕左上角显示第一个精灵
    SDL_Flip(gpScreen);
    */

    while (!quit)
    {
        while (SDL_PollEvent(&myEvent))
        {
            switch (myEvent.type)
            {
            case SDL_QUIT:
                quit = 1;
                break;
            case SDL_KEYDOWN:
                void handle_input(char *str, SDL_Surface *text);
                break;
            }
            case SDL_MOUSEBUTTONDOWN://鼠标按下
                  switch(myEvent.button.button)
                  {
                  case SDL_BUTTON_LEFT:
                      x = myEvent.button.x;//得到当前鼠标的坐标
                      y = myEvent.button.y;

                      //判断鼠标是否落在登录按钮里
                      if (isOnButton(x,y,0)==1)
                      {
                          selected=LOGIN;
                          changeButton(LEFT,2,&position[0]);
                          SDL_Flip(gpScreen);
                      }
                      else if (isOnButton(x,y,1)==1)//判断鼠标是否落在注册按钮里
                      {
                          selected=REGISTER;
                          changeButton(RIGHT,2,&position[1]);
                          SDL_Flip(gpScreen);
                      }
                      break;
                 }
                 break;
             case SDL_MOUSEBUTTONUP:
                 switch(myEvent.button.button)
                 {
                 case SDL_BUTTON_LEFT:
                     x = myEvent.button.x;//得到当前鼠标的坐标
                     y = myEvent.button.y;

                     //判断鼠标是否落在登录按钮里
                     if (selected==LOGIN)
                     {
                         changeButton(LEFT,1,&position[0]);
                         SDL_Flip(gpScreen);
                     }
                     else if (selected==REGISTER)//判断鼠标是否落在注册按钮里
                     {
                         changeButton(RIGHT,1,&position[1]);
                         SDL_Flip(gpScreen);
                     }
                     selected = OTHER;
                     break;
                 }
                 break;
             case SDL_MOUSEMOTION://鼠标移动
                 x = myEvent.button.x;//得到当前鼠标的坐标
                 y = myEvent.button.y;

                 //判断鼠标是否落在登录按钮里
                 if (isOnButton(x,y,0))
                 {
                     changeButton(LEFT,1,&position[0]);
                     changeButton(RIGHT,0,&position[1]);
                     SDL_Flip(gpScreen);
                 }
                 else if(isOnButton(x,y,1))
                 {
                     changeButton(LEFT,0,&position[0]);
                     changeButton(RIGHT,1,&position[1]);
                     SDL_Flip(gpScreen);
                 }
                 else
                 {
                     changeButton(LEFT,0,&position[0]);
                     changeButton(RIGHT,0,&position[1]);
                     SDL_Flip(gpScreen);
                 }
                 break;
            }
            SDL_BlitSurface(gpPLAYER,NULL,gpScreen,&dst);
            SDL_Flip(gpScreen);
        }


    SDL_FreeSurface(gpScreen);//退出程序前必须释放
    SDL_FreeSurface(gpBackGround);
    SDL_FreeSurface(gpPLAYER);
    SDL_Quit();

    return 0;

}

SDL_Surface *loadImage(char *aFilename) // 图像文件的名字 (return: SDL_Surface * 返回指向图像表面的指针) -->载入图像
{
    SDL_Surface* loadedImage = NULL;
    SDL_Surface* optimizedImage = NULL;

    //载入图像
    loadedImage = IMG_Load( aFilename);

    if( NULL != loadedImage )//If the image loaded
    {
        //创建优化图像
        optimizedImage = SDL_DisplayFormat( loadedImage );

        //释放loadImage
        SDL_FreeSurface( loadedImage );
    }
    return optimizedImage;
}

void cleanUp() //程序退出时，清理内存
{
     if (gpBackground)
     {
         SDL_FreeSurface(gpBackground);
     }
     if (gpButton)
     {
         SDL_FreeSurface(gpButton);
     }

     SDL_FreeSurface(gpScreen);
     SDL_Quit();
}

/*
参  数: (x,y)鼠标当前坐标;index要判断的那个按钮,0表示1，1表示2;
return: 如果落在指定按钮上返回1，否则返回0
--> 判断鼠标是否落在指定按钮上面
*/
int isOnButton(int aX,int aY, int aIndex)
{
     return aX>=position[aIndex].x && aX<=(position[aIndex].x+position[aIndex].w)&&
            aY>=position[aIndex].y && aY<=(position[aIndex].y + position[aIndex].h);
}


/*
参 数 : aPos表示要取精灵图左边还是右边的按钮;aNO要取的那个按钮，从上往下依次为0,1,2
--> 将指定按钮表面传输到显示表面
*/
void changeButton(BUTTONPOS aPos,int aNo,SDL_Rect *aDst)
{
     SDL_Rect src;

     switch(aPos)
     {
     case LEFT:
         src.x = 0;
         break;
     case RIGHT:
         src.x = BUTTONWIDTH + 1;
         break;
     }
     src.y =aNo? (aNo * BUTTONHEIGH + 1):(aNo * BUTTONHEIGH);
     src.w = BUTTONWIDTH;
     src.h = BUTTONHEIGH;
     SDL_BlitSurface(gpButton,&src,gpScreen,aDst);
}

BOOL init(char* aCaption,char * aIcon)
{
    //初始化 SDL
    if( SDL_Init( SDL_INIT_EVERYTHING ) == -1 )
    {
        return FALSE;
    }

    //载入程序图标
    SDL_WM_SetIcon(SDL_LoadBMP(aIcon), NULL);

    //初始化窗口
    gpScreen = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_SWSURFACE||SDL_HWSURFACE );
    if( gpScreen == NULL )//检测是否初始化成功
    {
        return FALSE;
    }
    atexit(cleanup);

    //初始化字体库
    if( TTF_Init() == -1 )
        return FALSE;

    //设置窗口名字和图标
    SDL_WM_SetCaption(localeToUTF8(aCaption), NULL );
    return TRUE;
}

char *localeToUTF8(char *src)
{
    static char *buf = NULL;
    wchar_t *unicode_buf;
    int nRetLen;

    if(buf){
        free(buf);
        buf = NULL;
    }
    nRetLen = MultiByteToWideChar(CP_ACP,0,src,-1,NULL,0);
    unicode_buf = (wchar_t*)malloc((nRetLen+1)*sizeof(wchar_t));
    MultiByteToWideChar(CP_ACP,0,src,-1,unicode_buf,nRetLen);
    nRetLen = WideCharToMultiByte(CP_UTF8,0,unicode_buf,-1,NULL,0,NULL,NULL);
    buf = (char*)malloc(nRetLen+1);
    WideCharToMultiByte(CP_UTF8,0,unicode_buf,-1,buf,nRetLen,NULL,NULL);
    free(unicode_buf);
    return buf;
}

wchar_t* cstringToUnicode(char * aSrc)
{
    int size;
    wchar_t *unicodestr = NULL;
    if(!aSrc)
    {
        return NULL;
    }
    size=MultiByteToWideChar(CP_ACP,0,aSrc,-1,NULL,0);
    unicodestr= malloc((size+1)*sizeof(wchar_t));
    MultiByteToWideChar(CP_ACP,0,aSrc,-1,unicodestr,size);
    return unicodestr;
}

/*
void end()
{
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();
}
*/


void handle_input(char *str, SDL_Surface *text)
{
    int n = strlen(str);

    if (event.type == SDL_KEYDOWN)
    {
        char *tmp = str;
        if (n <= 16)
        {
            if( event.key.keysym.unicode == (Uint16)' ' )
            {
                //在字符串末尾添加这个字符
                str[n] = (char)event.key.keysym.unicode;
                str[n+1] = 0;
            }
            //如果按键是数字
            else if( ( event.key.keysym.unicode >= (Uint16)'0' ) && ( event.key.keysym.unicode <= (Uint16)'9' ) )
            {
                //添加字符
                str[n] = (char)event.key.keysym.unicode;
                str[n+1] = 0;
            }
            //如果按键是一个大写字母
            else if( ( event.key.keysym.unicode >= (Uint16)'A' ) && ( event.key.keysym.unicode <= (Uint16)'Z' ) )
            {
                //添加字符
                str[n] = (char)event.key.keysym.unicode;
                str[n+1] = 0;
            }
            //如果按键是一个小写字符
            else if( ( event.key.keysym.unicode >= (Uint16)'a' ) && ( event.key.keysym.unicode <= (Uint16)'z' ) )
            {
                //添加字符
                str[n] = (char)event.key.keysym.unicode;
                str[n+1] = 0;
            }
        }
        //如果退格键被按下并且字符串不为空
        if( ( event.key.keysym.sym == SDLK_BACKSPACE ) && ( str.length() != 0 ) )
        {
            //从末尾移除一个字符
            str[n-1] = 0;
        }
        //如果字符串发生更改
        if( str != temp )
        {
            //释放旧的文本表面
            SDL_FreeSurface( text );

            //渲染一个新的文本表面
            text = TTF_RenderText_Solid( font, str, textColor );
        }
    }
}


void show_centered(SDL_Surface *text)
{
    //如果表面不是空的
    if( text != NULL )
    {
        //显示姓名
        apply_surface( ( SCREEN_WIDTH - text->w ) / 2, ( SCREEN_HEIGHT - text->h ) / 2, text, screen );
    }
}
